@@ -74,7 +74,7 @@ Main features
       >>> c = square(a)
       >>> # The above call did not trigger an evaluation
 
-2) **Embarrassingly parallel helper:** to make is easy to write readable
+2) **Embarrassingly parallel helper:** to make it easy to write readable
    parallel code and debug it quickly::
 
       >>> from joblib import Parallel, delayed
@@ -86,7 +86,7 @@ Main features
 3) **Logging/tracing:** The different functionalities will
    progressively acquire better logging mechanism to help track what
    has been ran, and capture I/O easily. In addition, Joblib will
-   provide a few I/O primitives, to easily define define logging and
+   provide a few I/O primitives, to easily define logging and
    display streams, and provide a way of compiling a report.
    We want to be able to quickly inspect what has been run.
 
@@ -115,7 +115,7 @@ Main features
 # Dev branch marker is: 'X.Y.dev' or 'X.Y.devN' where N is an integer.
 # 'X.Y.dev0' is the canonical version of 'X.Y.dev'
 #
-__version__ = '0.9.2'
+__version__ = '0.9.4'
 
 
 from .memory import Memory, MemorizedResult
