@@ -137,10 +137,7 @@ class SafeFunction(object):
             e_type, e_value, e_tb = sys.exc_info()
             text = format_exc(e_type, e_value, e_tb, context=10,
                               tb_offset=1)
-            if issubclass(e_type, TransportableException):
-                raise
-            else:
-                raise TransportableException(text, e_type)
+            raise TransportableException(text, e_type)
 
 
 ###############################################################################
@@ -533,7 +530,6 @@ class Parallel(Logger):
                     mmap_mode=self._mmap_mode,
                     temp_folder=self._temp_folder,
                     verbose=max(0, self.verbose - 50),
-                    context_id=0,  # the pool is used only for one call
                 )
                 if self._mp_context is not None:
                     # Use Python 3.4+ multiprocessing context isolation
@@ -751,14 +747,13 @@ Sub-process traceback:
                 # Kill remaining running processes without waiting for
                 # the results as we will raise the exception we got back
                 # to the caller instead of returning any result.
-                with self._lock:
-                    self._terminate_pool()
-                    if self._managed_pool:
-                        # In case we had to terminate a managed pool, let
-                        # us start a new one to ensure that subsequent calls
-                        # to __call__ on the same Parallel instance will get
-                        # a working pool as they expect.
-                        self._initialize_pool()
+                self._terminate_pool()
+                if self._managed_pool:
+                    # In case we had to terminate a managed pool, let
+                    # us start a new one to ensure that subsequent calls
+                    # to __call__ on the same Parallel instance will get
+                    # a working pool as they expect.
+                    self._initialize_pool()
                 raise exception
 
     def __call__(self, iterable):
@@ -799,10 +794,13 @@ Sub-process traceback:
         self.n_completed_tasks = 0
         self._smoothed_batch_duration = 0.0
         try:
-            self._iterating = True
-
+            # Only set self._iterating to True if at least a batch
+            # was dispatched. In particular this covers the edge
+            # case of Parallel used with an exhausted iterator.
             while self.dispatch_one_batch(iterator):
-                pass
+                self._iterating = True
+            else:
+                self._iterating = False
 
             if pre_dispatch == "all" or n_jobs == 1:
                 # The iterable was consumed all at once by the above for loop.
