@@ -20,15 +20,17 @@ import random
 
 from nose.tools import assert_equal
 
-from joblib.hashing import hash, PY3
+from joblib.hashing import hash
 from joblib.func_inspect import filter_args
 from joblib.memory import Memory
-
+from joblib.testing import assert_raises_regex
 from joblib.test.test_memory import env as test_memory_env
 from joblib.test.test_memory import setup_module as test_memory_setup_func
 from joblib.test.test_memory import teardown_module as test_memory_teardown_func
-
 from joblib.test.common import np, with_numpy
+from joblib.my_exceptions import TransportableException
+from joblib._compat import PY3_OR_LATER
+
 
 try:
     # Python 2/Python 3 compat
@@ -95,7 +97,13 @@ def test_trival_hash():
                 None,
                 gc.collect,
                 [1, ].append,
-               ]
+                # Next 2 sets have unorderable elements in python 3.
+                set(('a', 1)),
+                set(('a', 1, ('a', 1))),
+                # Next 2 dicts have unorderable type of keys in python 3.
+                {'a': 1, 1: 2},
+                {'a': 1, 1: 2, 'd': {'a': 1}},
+                ]
     for obj1 in obj_list:
         for obj2 in obj_list:
             # Check that 2 objects have the same hash only if they are
@@ -348,12 +356,6 @@ def test_dtype():
     assert_equal(hash([a, c]), hash([a, b]))
 
 
-class MyClass(object):
-    def __init__(self, a, b):
-        self.a, self.b = a, b
-        self.c = 'fixed'
-
-
 def test_hashes_stay_the_same():
     # We want to make sure that hashes don't change with joblib
     # version. For end users, that would mean that they have to
@@ -364,7 +366,8 @@ def test_hashes_stay_the_same():
                     u"C'est l\xe9t\xe9",
                     (123456, 54321, -98765),
                     [rng.random() for _ in range(5)],
-                    [3, 'abc', None, MyClass(1, 2)],
+                    [3, 'abc', None,
+                     TransportableException('the message', ValueError)],
                     {'abcde': 123, 'sadfas': [-9999, 2, 3]}]
 
     # These expected results have been generated with joblib 0.9.2
@@ -373,16 +376,16 @@ def test_hashes_stay_the_same():
                 '2ff3a25200eb6219f468de2640913c2d',
                 '50d81c80af05061ac4dcdc2d5edee6d6',
                 '536af09b66a087ed18b515acc17dc7fc',
-                'b5547baee3f205fb763e8a97c130c054',
+                '123ffc6f13480767167e171a8e1f6f4a',
                 'fc9314a39ff75b829498380850447047'],
         'py3': ['71b3f47df22cb19431d85d92d0b230b2',
                 '2d8d189e9b2b0b2e384d93c868c0e576',
                 'e205227dd82250871fa25aa0ec690aa3',
                 '9e4e9bf9b91890c9734a6111a35e6633',
-                '731fafc4405a6c192c0a85a58c9e7a93',
+                '6065a3c48e842ea5dee2cfd0d6820ad6',
                 'aeda150553d4bb5c69f0e69d51b0e2ef']}
 
-    py_version_str = 'py3' if PY3 else 'py2'
+    py_version_str = 'py3' if PY3_OR_LATER else 'py2'
     expected_list = expected_dict[py_version_str]
 
     for to_hash, expected in zip(to_hash_list, expected_list):
@@ -397,35 +400,44 @@ def test_hashes_stay_the_same_with_numpy_objects():
     # lengthy recomputations.
     rng = np.random.RandomState(42)
     # Being explicit about dtypes in order to avoid
-    # architecture-related differences
+    # architecture-related differences. Also using 'f4' rather than
+    # 'f8' for float arrays because 'f8' arrays generated by
+    # rng.random.randn don't seem to be bit-identical on 32bit and
+    # 64bit machines.
     to_hash_list = [
         rng.randint(-1000, high=1000, size=50).astype('<i8'),
-        tuple(rng.randn(3).astype('<f8') for _ in range(5)),
-        [rng.randn(3).astype('<f8') for _ in range(5)],
-        [3, 'abc', None, MyClass(1, 2)],
+        tuple(rng.randn(3).astype('<f4') for _ in range(5)),
+        [rng.randn(3).astype('<f4') for _ in range(5)],
         {
-            -3333: rng.randn(3, 5).astype('<f8'),
+            -3333: rng.randn(3, 5).astype('<f4'),
             0: [
                 rng.randint(10, size=20).astype('<i8'),
-                rng.randn(10).astype('<f8')
+                rng.randn(10).astype('<f4')
             ]
         }
     ]
 
     # These expected results have been generated with joblib 0.9.0
     expected_dict = {'py2': ['80f2387e7752abbda2658aafed49e086',
-                             '6028954a4c71750d74e66414bbab227b',
-                             'd32171e5bcf0e60034a7923bfea59e08',
-                             'b5547baee3f205fb763e8a97c130c054',
-                             '7989a9bbb5b7334efeda618843ab3ecc'],
+                             '0d700f7f25ea670fd305e4cd93b0e8cd',
+                             '83a2bdf843e79e4b3e26521db73088b9',
+                             '63e0efd43c0a9ad92a07e8ce04338dd3'],
                      'py3': ['10a6afc379ca2708acfbaef0ab676eab',
-                             'd93ceeb29ba96f03dfa3f846aa05f221',
-                             '54786bcda3d81ef61949d7a103168cee',
-                             '731fafc4405a6c192c0a85a58c9e7a93',
-                             '245de781f3b1948d4af0331d37a1e47f']}
+                             '988a7114f337f381393025911ebc823b',
+                             'c6809f4b97e35f2fa0ee8d653cbd025c',
+                             'b3ad17348e32728a7eb9cda1e7ede438']}
 
-    py_version_str = 'py3' if PY3 else 'py2'
+    py_version_str = 'py3' if PY3_OR_LATER else 'py2'
     expected_list = expected_dict[py_version_str]
 
     for to_hash, expected in zip(to_hash_list, expected_list):
         yield assert_equal, hash(to_hash), expected
+
+
+def test_hashing_pickling_error():
+    def non_picklable():
+        return 42
+
+    assert_raises_regex(pickle.PicklingError,
+                        'PicklingError while hashing',
+                        hash, non_picklable)
