@@ -10,7 +10,11 @@ import time
 import sys
 import io
 import os
+
 from joblib.test.common import np, with_numpy
+from joblib.test.common import with_multiprocessing
+from joblib.testing import check_subprocess_call
+from joblib._compat import PY3_OR_LATER
 
 try:
     import cPickle as pickle
@@ -20,7 +24,7 @@ except:
     PickleError = pickle.PicklingError
 
 
-if sys.version_info[0] == 3:
+if PY3_OR_LATER:
     PickleError = pickle.PicklingError
 
 try:
@@ -59,9 +63,17 @@ def square(x):
     return x ** 2
 
 
-def exception_raiser(x):
+class MyExceptionWithFinickyInit(Exception):
+    """An exception class with non trivial __init__
+    """
+    def __init__(self, a, b, c, d):
+        pass
+
+
+def exception_raiser(x, custom_exception=False):
     if x == 7:
-        raise ValueError
+        raise (MyExceptionWithFinickyInit('a', 'b', 'c', 'd')
+               if custom_exception else ValueError)
     return x
 
 
@@ -87,14 +99,14 @@ def test_cpu_count():
 def check_simple_parallel(backend):
     X = range(5)
     for n_jobs in (1, 2, -1, -2):
-        nose.tools.assert_equal(
+        assert_equal(
             [square(x) for x in X],
             Parallel(n_jobs=n_jobs)(delayed(square)(x) for x in X))
     try:
         # To smoke-test verbosity, we capture stdout
         orig_stdout = sys.stdout
         orig_stderr = sys.stdout
-        if sys.version_info[0] == 3:
+        if PY3_OR_LATER:
             sys.stderr = io.StringIO()
             sys.stderr = io.StringIO()
         else:
@@ -149,7 +161,7 @@ def test_mutate_input_with_threads():
     q = Queue(maxsize=5)
     Parallel(n_jobs=2, backend="threading")(
         delayed(q.put, check_pickle=False)(1) for _ in range(5))
-    nose.tools.assert_true(q.full())
+    assert_true(q.full())
 
 
 def test_parallel_kwargs():
@@ -258,6 +270,13 @@ def test_error_capture():
     # exception to make it easy to catch them
     assert_raises(ZeroDivisionError, Parallel(n_jobs=2),
                   [delayed(division)(x, y) for x, y in zip((0, 1), (1, 0))])
+
+    assert_raises(
+        MyExceptionWithFinickyInit,
+        Parallel(n_jobs=2, verbose=0),
+        (delayed(exception_raiser)(i, custom_exception=True)
+         for i in range(30)))
+
     try:
         # JoblibException wrapping is disabled in sequential mode:
         ex = JoblibException()
@@ -274,7 +293,7 @@ class Counter(object):
 
     def __call__(self, i):
         self.list1.append(i)
-        nose.tools.assert_equal(len(self.list1), len(self.list2))
+        assert_equal(len(self.list1), len(self.list2))
 
 
 def consumer(queue, item):
@@ -294,7 +313,7 @@ def check_dispatch_one_job(backend):
     # disable batching
     Parallel(n_jobs=1, batch_size=1, backend=backend)(
         delayed(consumer)(queue, x) for x in producer())
-    nose.tools.assert_equal(queue, [
+    assert_equal(queue, [
         'Produced 0', 'Consumed 0',
         'Produced 1', 'Consumed 1',
         'Produced 2', 'Consumed 2',
@@ -302,7 +321,7 @@ def check_dispatch_one_job(backend):
         'Produced 4', 'Consumed 4',
         'Produced 5', 'Consumed 5',
     ])
-    nose.tools.assert_equal(len(queue), 12)
+    assert_equal(len(queue), 12)
 
     # empty the queue for the next check
     queue[:] = []
@@ -310,7 +329,7 @@ def check_dispatch_one_job(backend):
     # enable batching
     Parallel(n_jobs=1, batch_size=4, backend=backend)(
         delayed(consumer)(queue, x) for x in producer())
-    nose.tools.assert_equal(queue, [
+    assert_equal(queue, [
         # First batch
         'Produced 0', 'Produced 1', 'Produced 2', 'Produced 3',
         'Consumed 0', 'Consumed 1', 'Consumed 2', 'Consumed 3',
@@ -318,7 +337,7 @@ def check_dispatch_one_job(backend):
         # Second batch
         'Produced 4', 'Produced 5', 'Consumed 4', 'Consumed 5',
     ])
-    nose.tools.assert_equal(len(queue), 12)
+    assert_equal(len(queue), 12)
 
 
 def test_dispatch_one_job():
@@ -349,9 +368,9 @@ def check_dispatch_multiprocessing(backend):
     # The the first consumption event can sometimes happen before the end of
     # the dispatching, hence, pop it before introspecting the "Produced" events
     first_four.remove('Consumed any')
-    nose.tools.assert_equal(first_four,
-                            ['Produced 0', 'Produced 1', 'Produced 2'])
-    nose.tools.assert_equal(len(queue), 12)
+    assert_equal(first_four,
+                 ['Produced 0', 'Produced 1', 'Produced 2'])
+    assert_equal(len(queue), 12)
 
 
 def test_dispatch_multiprocessing():
@@ -384,7 +403,7 @@ def test_batching_auto_multiprocessing():
 
 def test_exception_dispatch():
     "Make sure that exception raised during dispatch are indeed captured"
-    nose.tools.assert_raises(
+    assert_raises(
             ValueError,
             Parallel(n_jobs=2, pre_dispatch=16, verbose=0),
                     (delayed(exception_raiser)(i) for i in range(30)),
@@ -394,10 +413,10 @@ def test_exception_dispatch():
 def test_nested_exception_dispatch():
     # Ensure TransportableException objects for nested joblib cases gets
     # propagated.
-    nose.tools.assert_raises(
+    assert_raises(
         JoblibException,
         Parallel(n_jobs=2, pre_dispatch=16, verbose=0),
-                (delayed(SafeFunction(exception_raiser))(i) for i in range(30)))
+        (delayed(SafeFunction(exception_raiser))(i) for i in range(30)))
 
 
 def _reload_joblib():
@@ -417,8 +436,8 @@ def test_multiple_spawning():
     # systems that do not support fork
     if not int(os.environ.get('JOBLIB_MULTIPROCESSING', 1)):
         raise nose.SkipTest()
-    nose.tools.assert_raises(ImportError, Parallel(n_jobs=2, pre_dispatch='all'),
-                             [delayed(_reload_joblib)() for i in range(10)])
+    assert_raises(ImportError, Parallel(n_jobs=2, pre_dispatch='all'),
+                  [delayed(_reload_joblib)() for i in range(10)])
 
 
 ###############################################################################
@@ -434,7 +453,7 @@ def test_joblib_exception():
 
 def test_safe_function():
     safe_division = SafeFunction(division)
-    nose.tools.assert_raises(JoblibException, safe_division, 1, 0)
+    assert_raises(JoblibException, safe_division, 1, 0)
 
 
 def test_invalid_batch_size():
@@ -511,3 +530,52 @@ def test_no_blas_crash_or_freeze_with_multiprocessing():
     # in the worker processes managed by multiprocessing
     Parallel(n_jobs=2, backend=spawn_backend)(
         delayed(np.dot)(a, a.T) for i in range(2))
+
+
+def test_parallel_with_interactively_defined_functions():
+    # When functions are defined interactively in a python/IPython
+    # session, we want to be able to use them with joblib.Parallel
+
+    try:
+        import posix
+    except ImportError:
+        # This test pass only when fork is the process start method
+        raise nose.SkipTest('Not a POSIX platform')
+
+    code = '\n\n'.join([
+        'from joblib import Parallel, delayed',
+        'def sqrt(x): return x**2',
+        'print(Parallel(n_jobs=2)(delayed(sqrt)(i) for i in range(5)))'])
+
+    check_subprocess_call([sys.executable, '-c', code],
+                          stdout_regex=r'\[0, 1, 4, 9, 16\]')
+
+
+def test_parallel_with_exhausted_iterator():
+    exhausted_iterator = iter([])
+    assert_equal(Parallel(n_jobs=2)(exhausted_iterator), [])
+
+
+def check_memmap(a):
+    if not isinstance(a, np.memmap):
+        raise TypeError('Expected np.memmap instance, got %r',
+                        type(a))
+    return a.copy()  # return a regular array instead of a memmap
+
+
+@with_numpy
+@with_multiprocessing
+def test_auto_memmap_on_arrays_from_generator():
+    # Non-regression test for a problem with a bad interaction between the
+    # GC collecting arrays recently created during iteration inside the
+    # parallel dispatch loop and the auto-memmap feature of Parallel.
+    # See: https://github.com/joblib/joblib/pull/294
+    def generate_arrays(n):
+        for i in range(n):
+            yield np.ones(10, dtype=np.float32) * i
+    # Use max_nbytes=1 to force the use of memory-mapping even for small
+    # arrays
+    results = Parallel(n_jobs=2, max_nbytes=1)(
+        delayed(check_memmap)(a) for a in generate_arrays(100))
+    for result, expected in zip(results, generate_arrays(len(results))):
+        np.testing.assert_array_equal(expected, result)
